#include "MemoryReader.h"

#include <ArmarXCore/core/logging/Logging.h>

#include "../memory/ice_conversions.h"


namespace armarx::armem
{

    MemoryReader::MemoryReader(ReadingInterfacePrx memory) : memory(memory)
    {
    }


    MemoryPtr MemoryReader::getLatestSnapshots()
    {
        armem::data::query::EntityQueryPtr entityQuery = new armem::data::query::entity::Single();
        armem::data::query::ProviderSegmentQueryPtr providerQuery = new armem::data::query::provider::All({entityQuery});
        armem::data::query::CoreSegmentQueryPtr coreQuery = new armem::data::query::core::All({providerQuery});
        armem::data::query::MemoryQueryPtr memoryQuery = new armem::data::query::memory::All({coreQuery});

        return queryToCpp(memoryQuery);
    }

    MemoryPtr MemoryReader::getAllData()
    {
        armem::data::query::EntityQueryPtr entityQuery = new armem::data::query::entity::All();
        armem::data::query::ProviderSegmentQueryPtr providerQuery = new armem::data::query::provider::All({entityQuery});
        armem::data::query::CoreSegmentQueryPtr coreQuery = new armem::data::query::core::All({providerQuery});
        armem::data::query::MemoryQueryPtr memoryQuery = new armem::data::query::memory::All({coreQuery});

        return queryToCpp(memoryQuery);
    }

    data::MemoryPtr MemoryReader::query(data::query::MemoryQueryPtr query)
    {
        return this->query(data::query::MemoryQuerySeq{query});
    }

    data::MemoryPtr MemoryReader::query(const data::query::MemoryQuerySeq& queries)
    {
        data::QueryInput input;
        input.memoryQueries = queries;
        data::QueryResult result = this->query(input);

        if (result.success)
        {
            return result.memory;
        }
        else
        {
            ARMARX_ERROR << result.errorMessage;
            return nullptr;
        }
    }

    data::QueryResult MemoryReader::query(const data::QueryInput& input)
    {
        auto makeErrorMsg = [](auto e)
        {
            std::stringstream ss;
            ss << "Memory query failed.\nReason: " << e.what();
            return ss.str();
        };

        data::QueryResult result;
        try
        {
            result = memory->query(input);
        }
        catch (const Ice::ConnectionRefusedException& e)
        {
            result.errorMessage = makeErrorMsg(e);
        }
        catch (const Ice::ConnectionLostException& e)
        {
            result.errorMessage = makeErrorMsg(e);
        }
        catch (const Ice::NotRegisteredException& e)
        {
            result.errorMessage = makeErrorMsg(e);
        }
        return result;
    }

    MemoryPtr MemoryReader::queryToCpp(data::query::MemoryQueryPtr query)
    {
        return armem::fromIce<armem::MemoryPtr>(this->query(query));
    }

    MemoryPtr MemoryReader::queryToCpp(data::query::MemoryQuerySeq queries)
    {
        return armem::fromIce<armem::MemoryPtr>(this->query(queries));
    }

}
