#pragma once

#include <vector>

#include <Ice/Handle.h>

#include <RobotAPI/libraries/armem/core/MemoryID.h>


namespace armarx::armem::client::query::detail
{

    template <class _DerivedT, class _QueryT>
    class ChildSelectorOps
    {
    public:
        using DerivedT = _DerivedT;
        using QueryT = _QueryT;


        virtual DerivedT& all() = 0;
        virtual DerivedT& withID(const MemoryID& id) = 0;


    protected:

        template <class DerivedQueryT>
        DerivedQueryT& _addQuery()
        {
            IceInternal::Handle<DerivedQueryT> query(new DerivedQueryT);
            _queries.push_back(query);
            return *query;
        }

        std::vector<QueryT> _queries;

    };



    template <class _DerivedT, class _ChildT>
    class ParentSelectorOps
    {
    public:
        using DerivedT = _DerivedT;
        using ChildT = _ChildT;


    protected:

        ChildT& _addChild()
        {
            return _children.emplace_back();
        }
        ChildT& _addChild(const ChildT& child)
        {
            return _children.emplace_back(child);
        }

        std::vector<ChildT> _children;

    };



    template <class DerivedT, class QueryT, class ChildT>
    class InnerSelectorOps :
        public ParentSelectorOps<DerivedT, ChildT>,
        public ChildSelectorOps<DerivedT, QueryT>
    {
    public:

        virtual ~InnerSelectorOps()
        {}


        virtual std::vector<QueryT> buildQueries() const
        {
            std::vector<typename ChildT::QueryT> childQueries;
            for (const auto& child : this->_children)
            {
                for (const auto& query : child.buildQueries())
                {
                    childQueries.push_back(query);
                }
            }

            std::vector<QueryT> queries = this->_queries;
            for (auto& query : queries)
            {
                this->_setChildQueries(query, childQueries);
            }
            return queries;
        }


    protected:

        virtual void _setChildQueries(QueryT& query, const std::vector<typename ChildT::QueryT>& childQueries) const = 0;

    };

}
