#include "selectors.h"

#include <RobotAPI/libraries/armem/core/ice_conversions.h>


namespace dq = ::armarx::armem::data::query;


namespace armarx::armem::client::query
{
    dq::EntityQuerySeq SnapshotsSelector::buildQueries() const
    {
        return _queries;
    }

    SnapshotsSelector& SnapshotsSelector::all()
    {
        auto& q = _addQuery<dq::entity::All>();
        (void) q;
        return *this;
    }

    SnapshotsSelector& SnapshotsSelector::latest()
    {
        auto& q = _addQuery<dq::entity::Single>();
        q.timestamp = -1;
        return *this;
    }

    SnapshotsSelector& SnapshotsSelector::atTime(Time timestamp)
    {
        auto& q = _addQuery<dq::entity::Single>();
        toIce(q.timestamp, timestamp);
        return *this;
    }

    SnapshotsSelector& SnapshotsSelector::timeRange(Time min, Time max)
    {
        auto& q = _addQuery<dq::entity::TimeRange>();
        toIce(q.minTimestamp, min);
        toIce(q.maxTimestamp, max);
        return *this;
    }

    SnapshotsSelector& SnapshotsSelector::indexRange(long first, long last)
    {
        auto& q = _addQuery<dq::entity::IndexRange>();
        q.first = first;
        q.last = last;
        return *this;
    }



    SnapshotsSelector& EntitiesSelector::snapshots()
    {
        return _addChild();
    }

    SnapshotsSelector& EntitiesSelector::snapshots(const ChildT& selector)
    {
        return _addChild(selector);
    }

    EntitiesSelector& EntitiesSelector::all()
    {
        auto& q = _addQuery<dq::provider::All>();
        (void) q;
        return *this;
    }

    EntitiesSelector& EntitiesSelector::withName(const std::string& name)
    {
        auto& q = _addQuery<dq::provider::Single>();
        q.entityName = name;
        return *this;
    }

    EntitiesSelector& EntitiesSelector::withNameMatching(const std::string& regex)
    {
        auto& q = _addQuery<dq::provider::Regex>();
        q.entityNameRegex = regex;
        return *this;
    }

    void EntitiesSelector::_setChildQueries(dq::ProviderSegmentQueryPtr& query, const dq::EntityQuerySeq& childQueries) const
    {
        query->entityQueries = childQueries;
    }



    EntitiesSelector& ProviderSegmentSelector::entities()
    {
        return _addChild();
    }

    EntitiesSelector& ProviderSegmentSelector::entities(const EntitiesSelector& selector)
    {
        return _addChild(selector);
    }

    ProviderSegmentSelector& ProviderSegmentSelector::all()
    {
        auto& q = _addQuery<dq::core::All>();
        (void) q;
        return *this;
    }

    ProviderSegmentSelector& ProviderSegmentSelector::withName(const std::string& name)
    {
        auto& q = _addQuery<dq::core::Single>();
        q.providerSegmentName = name;
        return *this;
    }

    ProviderSegmentSelector& ProviderSegmentSelector::withNameMatching(const std::string& regex)
    {
        auto& q = _addQuery<dq::core::Regex>();
        q.providerSegmentNameRegex = regex;
        return *this;
    }

    void ProviderSegmentSelector::_setChildQueries(dq::CoreSegmentQueryPtr& query, const dq::ProviderSegmentQuerySeq& childQueries) const
    {
        query->providerSegmentQueries = childQueries;
    }



    ProviderSegmentSelector& CoreSegmentSelector::providerSegments()
    {
        return _addChild();
    }

    ProviderSegmentSelector& CoreSegmentSelector::providerSegments(const ProviderSegmentSelector& selector)
    {
        return _addChild(selector);
    }

    CoreSegmentSelector& CoreSegmentSelector::all()
    {
        auto& q = _addQuery<dq::memory::All>();
        (void) q;
        return *this;
    }

    CoreSegmentSelector& CoreSegmentSelector::withName(const std::string& name)
    {
        auto& q = _addQuery<dq::memory::Single>();
        q.coreSegmentName = name;
        return *this;
    }
    CoreSegmentSelector& CoreSegmentSelector::withNameMatching(const std::string& regex)
    {
        auto& q = _addQuery<dq::memory::Regex>();
        q.coreSegmentNameRegex = regex;
        return *this;
    }

    void CoreSegmentSelector::_setChildQueries(dq::MemoryQueryPtr& query, const dq::CoreSegmentQuerySeq& childQueries) const
    {
        query->coreSegmentQueries = childQueries;
    }

}
