#pragma once

#include <RobotAPI/interface/armem/query.h>

#include <RobotAPI/libraries/armem/core/Time.h>

#include "NameSelectorOps.h"
#include "SelectorOps.h"


namespace armarx::armem::client::query
{

    class SnapshotsSelector :
        public detail::ChildSelectorOps<SnapshotsSelector, data::query::EntityQueryPtr>
    {
    public:

        data::query::EntityQuerySeq buildQueries() const;


        SnapshotsSelector& all() override;
        SnapshotsSelector& withID(const MemoryID& id) override
        {
            return atTime(id.timestamp);
        }

        SnapshotsSelector& latest();
        SnapshotsSelector& atTime(Time timestamp);

        SnapshotsSelector& timeRange(Time min, Time max);
        SnapshotsSelector& indexRange(long first, long last);

    };



    class EntitiesSelector :
        public detail::InnerSelectorOps<EntitiesSelector, data::query::ProviderSegmentQueryPtr, SnapshotsSelector>
        , public detail::NameSelectorOps<EntitiesSelector>

    {
    public:

        /// Start specifying entity snapshots.
        SnapshotsSelector& snapshots();
        SnapshotsSelector& snapshots(const SnapshotsSelector& selector);


        EntitiesSelector& all() override;
        EntitiesSelector& withID(const MemoryID& id) override
        {
            return withName(id.entityName);
        }

        EntitiesSelector& withName(const std::string& name) override;
        EntitiesSelector& withNameMatching(const std::string& regex) override;
        using NameSelectorOps::withNames;


    protected:
        void _setChildQueries(data::query::ProviderSegmentQueryPtr& query, const data::query::EntityQuerySeq& childQueries) const override;
    };



    class ProviderSegmentSelector :
        public detail::InnerSelectorOps<ProviderSegmentSelector, data::query::CoreSegmentQueryPtr, EntitiesSelector>
        , public detail::NameSelectorOps<ProviderSegmentSelector>
    {
    public:

        /// Start specifying entities.
        EntitiesSelector& entities();
        EntitiesSelector& entities(const EntitiesSelector& selector);


        ProviderSegmentSelector& all() override;
        ProviderSegmentSelector& withID(const MemoryID& id) override
        {
            return withName(id.providerSegmentName);
        }

        ProviderSegmentSelector& withName(const std::string& name) override;
        ProviderSegmentSelector& withNameMatching(const std::string& regex) override;
        using NameSelectorOps::withNames;


    protected:
        void _setChildQueries(data::query::CoreSegmentQueryPtr& query, const data::query::ProviderSegmentQuerySeq& childQueries) const override;
    };



    class CoreSegmentSelector :
        public detail::InnerSelectorOps<CoreSegmentSelector, data::query::MemoryQueryPtr, ProviderSegmentSelector>
        , public detail::NameSelectorOps<CoreSegmentSelector>
    {
    public:

        /// Start specifying provider segments.
        ProviderSegmentSelector& providerSegments();
        ProviderSegmentSelector& providerSegments(const ProviderSegmentSelector& selector);


        CoreSegmentSelector& all() override;
        CoreSegmentSelector& withID(const MemoryID& id) override
        {
            return withName(id.coreSegmentName);
        }

        CoreSegmentSelector& withName(const std::string& name) override;
        CoreSegmentSelector& withNameMatching(const std::string& regex) override;
        using NameSelectorOps::withNames;


    protected:
        void _setChildQueries(data::query::MemoryQueryPtr& query, const data::query::CoreSegmentQuerySeq& childQueries) const override;
    };

}
