#include "EntityInstance.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>


namespace armarx::armem
{

    EntityInstance::EntityInstance()
    {
    }

    EntityInstance::EntityInstance(int index, const MemoryID& parentID) :
        EntityInstance(parentID.withInstanceIndex(index))
    {
    }

    EntityInstance::EntityInstance(const MemoryID& id) : MemoryItem(id)
    {
    }

    EntityInstance::~EntityInstance()
    {
    }

    void EntityInstance::setData(aron::data::AronDataPtr data)
    {
        this->_data = data;
        try
        {
            this->_dataNavigator = armarx::aron::datanavigator::AronDataNavigator::FromAronData(this->_data);
        }
        catch (const armarx::LocalException&)
        {
            this->_dataNavigator = nullptr;
        }
    }

    void EntityInstance::update(const InternalEntityUpdate& update, int index)
    {
        ARMARX_CHECK_FITS_SIZE(index, update.instancesData.size());

        this->index() = index;
        setData(update.instancesData.at(size_t(index)));

        this->_metadata.confidence = update.confidence;

        this->_metadata.timeCreated = update.timeCreated;
        this->_metadata.timeSent = update.timeSent;
        this->_metadata.timeArrived = update.timeArrived;
    }

    EntityInstancePtr EntityInstance::copy(bool withData) const
    {
        EntityInstancePtr other = std::make_unique<EntityInstance>();
        other->id() = id();
        other->_metadata = _metadata;
        if (withData)
        {
            other->setData(_data);
        }
        return other;
    }

    std::string EntityInstance::getKeyString() const
    {
        return std::to_string(index());
    }

    std::string EntityInstance::getLevelName() const
    {
        return "entity instance";
    }

}
