#pragma once

#include <memory>

#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/aroncore/navigators/datanavigator/AronDataNavigator.h>

#include "../core/Time.h"

#include "InternalCommit.h"
#include "detail/MemoryItem.h"


namespace armarx::armem
{

    /**
     * @brief Metadata of an entity instance.
     */
    struct EntityInstanceMetadata
    {
        /// Time when this value was created.
        Time timeCreated;
        /// Time when this value was sent to the memory.
        Time timeSent;
        /// Time when this value has arrived at the memory.
        Time timeArrived;

        /// An optional confidence, may be used for things like decay.
        float confidence = 1.0;
    };


    class EntityInstance;
    using EntityInstancePtr = std::unique_ptr<EntityInstance>;

    /**
     * @brief Data of a single entity instance.
     */
    class EntityInstance : public detail::MemoryItem
    {
    public:

        EntityInstance();
        EntityInstance(int index, const MemoryID& parentID = {});
        EntityInstance(const MemoryID& id);

        virtual ~EntityInstance() override;


        inline int& index()
        {
            return id().instanceIndex;
        }
        inline int index() const
        {
            return id().instanceIndex;
        }

        EntityInstanceMetadata& metadata()
        {
            return _metadata;
        }
        inline const EntityInstanceMetadata& metadata() const
        {
            return _metadata;
        }

        inline armarx::aron::data::AronDataPtr data() const
        {
            return _data;
        }
        void setData(aron::data::AronDataPtr data);

        inline armarx::aron::datanavigator::AronDataNavigatorPtr dataNavigator() const
        {
            return _dataNavigator;
        }


        /**
         * @brief Fill `*this` with the update's values.
         * @param update The update.
         * @param index The instances index.
         */
        void update(const InternalEntityUpdate& update, int index);


        EntityInstancePtr copy(bool withData = true) const;


        // MemoryItem interface
    public:
        std::string getKeyString() const override;
        std::string getLevelName() const override;


    private:

        EntityInstanceMetadata _metadata;

        /// The actual data.
        armarx::aron::data::AronDataPtr _data;
        /// Allows investigating `data`'s structure. May be nullptr.
        armarx::aron::datanavigator::AronDataNavigatorPtr _dataNavigator;

    };

}
