#include "ice_conversions.h"


namespace armarx
{

    void armem::fromIce(const data::Commit& ice, InternalCommit& commit, Time timeArrived)
    {
        commit.updates.clear();
        for (const auto& ice_update : ice.updates)
        {
            InternalEntityUpdate& update = commit.updates.emplace_back(InternalEntityUpdate());
            fromIce(ice_update, update, timeArrived);
        }
    }


    void armem::fromIce(const data::EntityUpdate& ice, InternalEntityUpdate& update, Time timeArrived)
    {
        armem::fromIce(ice, update);
        update.timeArrived = timeArrived;
    }


    void armem::detail::toIceItem(data::detail::MemoryItem& ice, const MemoryItem& item)
    {
        toIce(ice.id, item.id());
    }

    void armem::detail::fromIceItem(const data::detail::MemoryItem& ice, MemoryItem& item)
    {
        fromIce(ice.id, item.id());
    }


    void armem::toIce(data::EntityInstanceMetadata& ice, const EntityInstanceMetadata& metadata)
    {
        ice.confidence = metadata.confidence;
        toIce(ice.timeArrivedMicroSeconds, metadata.timeArrived);
        toIce(ice.timeCreatedMicroSeconds, metadata.timeCreated);
        toIce(ice.timeSentMicroSeconds, metadata.timeSent);
    }
    void armem::fromIce(const data::EntityInstanceMetadata& ice, EntityInstanceMetadata& metadata)
    {
        metadata.confidence = ice.confidence;
        fromIce(ice.timeArrivedMicroSeconds, metadata.timeArrived);
        fromIce(ice.timeCreatedMicroSeconds, metadata.timeCreated);
        fromIce(ice.timeSentMicroSeconds, metadata.timeSent);
    }

    void armem::toIce(data::EntityInstance& ice, const EntityInstance& data)
    {
        detail::toIceItem(ice, data);

        ice.data = data.data();
        toIce(ice.metadata, data.metadata());
    }
    void armem::fromIce(const data::EntityInstance& ice, EntityInstance& data)
    {
        detail::fromIceItem(ice, data);

        data.setData(ice.data);
        fromIce(ice.metadata, data.metadata());
    }


    void armem::toIce(data::EntitySnapshot& ice, const EntitySnapshot& snapshot)
    {
        detail::toIceItem(ice, snapshot);

        toIce(ice.instances, snapshot.instances);
    }
    void armem::fromIce(const data::EntitySnapshot& ice, EntitySnapshot& snapshot)
    {
        detail::fromIceItem(ice, snapshot);

        fromIce(ice.instances, snapshot.instances);
    }

    void armem::toIce(data::Entity& ice, const Entity& entity)
    {
        detail::toIceItem(ice, entity);

        toIce(ice.history, entity.history);
    }
    void armem::fromIce(const data::Entity& ice, Entity& entity)
    {
        detail::fromIceItem(ice, entity);

        fromIce(ice.history, entity.history);
    }


    void armem::toIce(data::ProviderSegment& ice, const ProviderSegment& providerSegment)
    {
        detail::toIceItem(ice, providerSegment);

        ice.aronType = providerSegment.aronType();
        toIce(ice.entities, providerSegment.entities);
    }
    void armem::fromIce(const data::ProviderSegment& ice, ProviderSegment& providerSegment)
    {
        detail::fromIceItem(ice, providerSegment);

        providerSegment.aronType() = ice.aronType;
        fromIce(ice.entities, providerSegment.entities);
    }

    void armem::toIce(data::CoreSegment& ice, const CoreSegment& coreSegment)
    {
        detail::toIceItem(ice, coreSegment);

        ice.aronType = coreSegment.aronType();
        toIce(ice.providerSegments, coreSegment.providerSegments);
    }
    void armem::fromIce(const data::CoreSegment& ice, CoreSegment& coreSegment)
    {
        detail::fromIceItem(ice, coreSegment);

        coreSegment.aronType() = ice.aronType;
        fromIce(ice.providerSegments, coreSegment.providerSegments);
    }

    void armem::toIce(data::Memory& ice, const Memory& memory)
    {
        detail::toIceItem(ice, memory);

        toIce(ice.coreSegments, memory.coreSegments);
    }
    void armem::fromIce(const data::Memory& ice, Memory& memory)
    {
        detail::fromIceItem(ice, memory);

        fromIce(ice.coreSegments, memory.coreSegments);
    }

}

