#pragma once

#include <map>

#include <QObject>

#include <RobotAPI/libraries/armem/memory/Memory.h>
#include <RobotAPI/libraries/armem_gui/TreeWidgetBuilder.h>


class QTreeWidget;


namespace armarx::armem
{

    class MemoryTreeWidget : public QTreeWidget
    {
        Q_OBJECT
    public:

        MemoryTreeWidget();

        void update(const armem::Memory& memory);
        void update(const std::map<std::string, const armem::Memory*>& memories);


    signals:

        void updated();
        void instanceSelected(const MemoryID& id);


    private slots:

        void handleSelection();


    private:

        void initWidget();
        void initBuilders();

        void updateChildren(const armem::Memory& memory, QTreeWidget* tree);
        void updateChildren(const std::map<std::string, const armem::Memory*>& memories, QTreeWidget* tree);

        void updateChildren(const armem::Memory& memory, QTreeWidgetItem* memoryItem);
        void updateChildren(const armem::CoreSegment& coreSeg, QTreeWidgetItem* coreSegItem);
        void updateChildren(const armem::ProviderSegment& provSeg, QTreeWidgetItem* provSegItem);
        void updateChildren(const armem::Entity& entity, QTreeWidgetItem* entityItem);
        void updateChildren(const armem::EntitySnapshot& snapshot, QTreeWidgetItem* snapshotItem);
        void updateChildren(const armem::EntityInstance& data, QTreeWidgetItem* parent);


        QTreeWidgetItem* makeItem(const std::string& key, const armem::detail::MemoryItem* container);

        void updateItemItem(const armem::detail::MemoryItem* level, QTreeWidgetItem* item);
        template <class... T>
        void updateContainerItem(const armem::detail::MemoryContainer<T...>* container, QTreeWidgetItem* item);
        template <class... T>
        void updateTypedContainerItem(const armem::detail::TypedEntityContainer<T...>* container, QTreeWidgetItem* item);


    private:

        bool instanceIntrospection = false;

        MapTreeWidgetBuilder<std::string, const Memory*> memoryBuilder;
        MapTreeWidgetBuilder<std::string, CoreSegmentPtr> coreSegmentBuilder;
        MapTreeWidgetBuilder<std::string, ProviderSegmentPtr> provSegmentBuilder;
        MapTreeWidgetBuilder<std::string, EntityPtr> entityBuilder;
        MapTreeWidgetBuilder<armem::Time, EntitySnapshotPtr> snapshotBuilder;
        TreeWidgetBuilder<std::vector<EntityInstancePtr>> instanceBuilder;


        enum class Columns
        {
            KEY = 0,
            SIZE = 1,
            TYPE = 2,
            LEVEL = 3,
            ID = 4,
            NUM = 5
        };
    };

}
