/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL

// ArmarX
#include <ArmarXCore/core/exceptions/Exception.h>
#include <RobotAPI/libraries/aron/aroncore/AronPath.h>
#include <RobotAPI/libraries/aron/aroncore/AronDescriptor.h>

namespace armarx
{
    namespace aron
    {
        namespace exception
        {
            class AronException :
                public armarx::LocalException
            {
            public:
                AronException() = delete;
                AronException(const std::string& caller, const std::string& method, const std::string& reason) :
                    LocalException(caller + "::" + method + ": " + reason)
                {

                }
            };

            class AronNotImplementedYetException :
                public armarx::LocalException
            {
            public:
                AronNotImplementedYetException() = delete;
                AronNotImplementedYetException(const std::string& caller, const std::string& method) :
                    LocalException(caller + "::" + method + ": This method is not yet implemented!")
                {

                }
            };

            class AronExceptionWithPathInfo :
                public armarx::LocalException
            {
            public:
                AronExceptionWithPathInfo() = delete;
                AronExceptionWithPathInfo(const std::string& caller, const std::string& method, const std::string& reason, const AronPath& path) :
                    LocalException(caller + "::" + method + ": " + reason + ". The path was: " + path.toString())
                {

                }
            };

            class AronDataNotValidException :
                public armarx::LocalException
            {
            public:
                AronDataNotValidException() = delete;
                AronDataNotValidException(const std::string& caller, const std::string& method, const std::string& reason, const data::AronDataPtr& data) :
                    LocalException(caller + "::" + method + ": " + reason + ". The ice_id of the data was: " + data->ice_id())
                {

                }
            };

            class AronTypeNotValidException :
                public armarx::LocalException
            {
            public:
                AronTypeNotValidException() = delete;
                AronTypeNotValidException(const std::string& caller, const std::string& method, const std::string& reason, const type::AronTypePtr& type) :
                    LocalException(caller + "::" + method + ": " + reason + ". The ice_id of the type was: " + type->ice_id())
                {

                }
            };

            class AronTypeDescriptorNotValidException :
                public armarx::LocalException
            {
            public:
                AronTypeDescriptorNotValidException() = delete;
                AronTypeDescriptorNotValidException(const std::string& caller, const std::string& method, const std::string& reason, const AronTypeDescriptor& type) :
                    LocalException(caller + "::" + method + ": " + reason + ". The name of the type-enum was: " + AronTypeDescriptorToString.at(type))
                {

                }
            };

            class AronDataDescriptorNotValidException :
                public armarx::LocalException
            {
            public:
                AronDataDescriptorNotValidException() = delete;
                AronDataDescriptorNotValidException(const std::string& caller, const std::string& method, const std::string& reason, const AronDataDescriptor& data) :
                    LocalException(caller + "::" + method + ": " + reason + ". The name of the data-enum was: " + AronDataDescriptorToString.at(data))
                {

                }
            };

            class AronStringNotValidExceptionException :
                public armarx::LocalException
            {
            public:
                AronStringNotValidExceptionException() = delete;
                AronStringNotValidExceptionException(const std::string& caller, const std::string& method, const std::string& reason, const std::string& input) :
                    LocalException(caller + "::" + method + ": " + reason + ". The expected string was: " + input)
                {

                }
            };

            class AronSizeNotValidExceptionException :
                public armarx::LocalException
            {
            public:
                AronSizeNotValidExceptionException() = delete;
                AronSizeNotValidExceptionException(const std::string& caller, const std::string& method, const std::string& reason, int real, int expected) :
                    LocalException(caller + "::" + method + ": " + reason + ". The measured size is: " + std::to_string(real) + ". The expected size was: " + std::to_string(expected))
                {

                }
            };
        }
    }
}
