/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <string>
#include <unordered_map>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/aroncore/AronConcepts.h>
#include <RobotAPI/libraries/aron/aroncore/AronPath.h>
#include <RobotAPI/libraries/aron/aroncore/AronException.h>

namespace armarx
{
    namespace aron
    {
        template <typename Input, typename Output>
        class AronFactory
        {
        public:
            AronFactory() = default;

            virtual Output create(const Input&, const AronPath&) const = 0;
            virtual Output createSpecific(const Input&, const AronPath&) const = 0;
        };

        template <typename Input, typename Output>
        class AronPtrInputFactory :
                virtual public AronFactory<Input, Output>
        {
        public:
            AronPtrInputFactory() = default;

            virtual Output create(const Input&, const AronPath&) const = 0;
            virtual Output createSpecific(const Input&, const AronPath&) const = 0;

        protected:
            static void CheckIfInputIsNull(const std::string& c, const std::string& m, const AronPath& p, const Input& i)
            {
                if(i.get() == nullptr)
                {
                    throw exception::AronExceptionWithPathInfo(c, m, "The used input is NULL", p);
                }
            }
        };

        template <typename Input, typename Output>
        class AronPtrOutputFactory :
                virtual public AronFactory<Input, Output>
        {
        public:
            AronPtrOutputFactory() = default;

            virtual Output create(const Input&, const AronPath&) const = 0;
            virtual Output createSpecific(const Input&, const AronPath&) const = 0;

        protected:
            static void CheckIfOutputIsNull(const std::string& c, const std::string& m, const AronPath& p, const Output& o)
            {
                if(o.get() == nullptr)
                {
                    throw exception::AronExceptionWithPathInfo(c, m, "The used output is NULL", p);
                }
            }
        };

        template <typename Input, typename Output>
        class AronPtrInputPtrOutputFactory :
                virtual public AronPtrInputFactory<Input, Output>,
                virtual public AronPtrOutputFactory<Input, Output>
        {
        public:
            AronPtrInputPtrOutputFactory() = default;

            virtual Output create(const Input&, const AronPath&) const = 0;
            virtual Output createSpecific(const Input&, const AronPath&) const = 0;
        };
    }
}
