#include "AronDataVisitor.h"

#include <SimoxUtility/meta/type_name.h>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>


namespace armarx::aron::visitor
{


    AronDataVisitor::AronDataVisitor()
    {
    }

    AronDataVisitor::~AronDataVisitor()
    {
    }


    bool AronDataVisitor::applyTo(data::AronDataPtr& data)
    {
        ARMARX_CHECK_NOT_NULL(data);
        return applyTo(*AronDataNavigator::FromAronData(data));
    }

    bool AronDataVisitor::applyTo(AronDataVisitor::AronDataNavigator& navigator)
    {
        return this->applyTo("", navigator);
    }

    bool AronDataVisitor::applyTo(const std::string& key, AronDataNavigator& navigator)
    {
        auto handleCollection = [this, key](auto & n)
        {
            if (!visitEnter(key, n))
            {
                return false;
            }
            if (!applyToChildren(n))
            {
                return false;
            }
            if (!visitExit(key, n))
            {
                return false;
            }
            return true;
        };

        if (auto n = dynamic_cast<AronDictDataNavigator*>(&navigator))
        {
            return handleCollection(*n);
        }
        else if (auto n = dynamic_cast<AronListDataNavigator*>(&navigator))
        {
            return handleCollection(*n);
        }

        ARMARX_CHECK_EQUAL(navigator.childrenSize(), 0)
                << simox::meta::get_type_name(navigator);

        if (auto n = dynamic_cast<AronBoolDataNavigator*>(&navigator))
        {
            return visit(key, *n);
        }
        else if (auto n = dynamic_cast<AronDoubleDataNavigator*>(&navigator))
        {
            return visit(key, *n);
        }
        else if (auto n = dynamic_cast<AronFloatDataNavigator*>(&navigator))
        {
            return visit(key, *n);
        }
        else if (auto n = dynamic_cast<AronIntDataNavigator*>(&navigator))
        {
            return visit(key, *n);
        }
        else if (auto n = dynamic_cast<AronLongDataNavigator*>(&navigator))
        {
            return visit(key, *n);
        }
        else if (auto n = dynamic_cast<AronNDArrayDataNavigator*>(&navigator))
        {
            return visit(key, *n);
        }
        else if (auto n = dynamic_cast<AronStringDataNavigator*>(&navigator))
        {
            return visit(key, *n);
        }
        ARMARX_CHECK(false) << "Unhandled AronDataNavigatorType '" << simox::meta::get_type_name(navigator) << "'.";
    }

    bool AronDataVisitor::applyToChildren(AronDataVisitor::AronDictDataNavigator& navigator)
    {
        for (const std::string& key : navigator.getAllKeys())
        {
            auto child = navigator.getElement(key);
            if (child)
            {
                if (!applyTo(key, *child))
                {
                    return false;
                }
            }
        }
        return true;
    }

    bool AronDataVisitor::applyToChildren(AronDataVisitor::AronListDataNavigator& navigator)
    {
        size_t i = 0;
        for (auto& child : navigator.getChildren())
        {
            if (child)
            {
                if (!applyTo(std::to_string(i), *child))
                {
                    return false;
                }
            }
            ++i;
        }
        return true;
    }

}
